var svg = d3.select("svg"),
width = +svg.attr("width"),
height = +svg.attr("height");

var node,
r=12,
color = d3.scaleOrdinal()
    .domain(["citing", "input", "cited"])
    .range([' #01d7c0', ' #8b90fe ', '  #a15eb2 ']),
yscale = d3.scaleOrdinal()
    .domain(["citing", "input", "cited"])
    .range([0, 200, 400]),
toRemove;

var link;

/**
* create a background
*/
var rect = svg.append("rect")
    .attr("x", 0)
    .attr("y", 0)
    .attr("height", height)
    .attr("width", width)
    .style("fill", 'white')
    .on('click', function(d) {
    d3.selectAll(".circle").style("stroke", "none")
    d3.selectAll(".link")
        .style("stroke", "#999")
        .attr('marker-end',marker('#999'))
    document.getElementById('textbox').innerHTML = "Click node";
});

var simulation = d3.forceSimulation()
    .force("link", d3.forceLink().id(function(d) {return d.doi;}).distance(100).strength(1))
    .force("collide", d3.forceCollide(50))
    .force("charge", d3.forceManyBody().strength(-30))
    .force("center", d3.forceCenter(width/2, height/2))
    .force("yscale", d3.forceY().strength(1).y(function(d) {return yscale(d.group)}));

/**
* create a new zoom behavior and apply it to svg
*/
var zoom = d3.zoom()
   .on("zoom", zoom);
zoom(svg);  

/**
* create group element
*/
var g = svg.append("g")
    .attr("class", "everything")

//d3.json("data.json", function (error, graph) {
d3.json("json_text.json").then(function(graph) {
    update(graph.links, graph.nodes);
})

function update(links, nodes) {
    link = g.append("g")
        .selectAll(".link")
        .data(links)
        .enter()
        .append("line")
        .style("stroke-width", "1px")
        .style("stroke", "#999")
        .attr("class", "link")
        .attr('marker-end',marker("#999"))

    node = g.append("g")       
        .selectAll(".node")
        .data(nodes)
        .enter()
        .append("g")
        .attr("class", "node")
        .call(d3.drag()
            .on("start", dragstarted)
            .on("drag", dragged)
        )
        .attr("initial_x", function(d) {return d.dx;})
        .attr("initial_y", function(d) {return d.dy;});

    node.append("circle")
        .attr("r", r)
        .attr("class", "circle")
        .style("fill", function(d){ return color(d.group)})
        .on('click', function (d) {
            if(toRemove){
                d3.select(toRemove).selectAll(".circle").style("stroke","none")
            }
            toRemove = this.parentNode;
            d3.select(this).style("stroke","black")
            marklink(d)
            textfunc(d)
        });

    node.append("text")
        .attr("class", "text") //über selectAll(".text") können objs mit der klasse ausgewählt werden
        .style("font-size", "15px")
        .text(function (d) {return firstauthor(d.author);})
        .style('pointer-events', 'auto')
        .on('click', function(d) {
           if(toRemove){
           d3.select(toRemove).selectAll(".circle").style("stroke","none");
        }
        toRemove = this.parentNode;
        d3.select(this.parentNode).selectAll(".circle").style("stroke","black")

        marklink(d)
        textfunc(d);
    });

simulation
    .nodes(nodes)
    .on("tick", ticked);

simulation.force("link")
    .links(links);
}

function isLinkForNode(node, link){
return link.source.index == node.index || link.target.index == node.index;
}

function marklink(node){
    d3.selectAll(".link")
        .style("stroke", function(o) {
            return isLinkForNode(node, o) ? "black" : "#999";})
        .attr('marker-end', function(o) {
            return isLinkForNode(node, o) ? marker('#000000') : marker('#999');})
}

function marker(color) {
    svg.append('defs').append('marker')//arrowhead
        .attr('id',color.replace("#", ""))
        .attr('viewBox','-0 -5 10 10')
        .attr('refX',r+10)
        .attr('refY',0)
        .attr('orient','auto')
        .attr('markerWidth',10)
        .attr('markerHeight',15)
        .attr('xoverflow','visible')
        .append('svg:path')
        .attr('d', 'M 0,-5 L 10 ,0 L 0,5')
        .attr('fill', color)//arrowhead color
        .style('stroke','none');
    return "url(" + color + ")";
};

/**
* returns last name of first author
* @param {string} authors - The comma-separated string of authors
*/
function firstauthor(authors){
    if (/,/.test(authors)==false){
        var firstauthor=/^.*\s+([\w\-]+)[\.\s]*$/.exec(authors)
    }
    else {
        var firstauthor=/^[\s\w\.\-]*\s([\w\-]+)[\.\s]*,.*$/.exec(authors)
    }
    return firstauthor[1]
}

function textfunc(d){
    document.getElementById('textbox').innerHTML = "Title:" + '</br>' + d.name +
    '</br>' +'</br>'+"Author:"+ '</br>' +d.author+'</br>'+'</br>'+"Year:"+'</br>'+d.year+'</br>'+'</br>'
    +"doi:"+'</br>'+d.doi;
}

function ticked() {
    link.attr("x1", function (d) {return d.source.x;})
        .attr("y1", function (d) {return d.source.y;})
        .attr("x2", function (d) {return d.target.x;})
        .attr("y2", function (d) {return d.target.y;});
    node.attr("transform", function (d) {return "translate(" + d.x + ", " + d.y + ")";});
}

function dragstarted(d) {
    if (!d3.event.active) 
        simulation.alphaTarget(0.3).restart()
    d.fx = d.x;
    d.fy = d.y;
}

function dragged(d) {
    d.fx = d3.event.x;
    d.fy = d3.event.y;
}

function resetGraph() {
    d3.selectAll(".node").each(function(d) {
        d.fx = d.initial_x;
        d.fy = d.initial_y;
    })
}

function zoom() {
    d3.select('g').attr("transform", d3.event.transform);
}

/**
* transforms svg such that the zoom is reset
*/
function resetZoom() {
    d3.select('svg')
        .call(zoom.scaleTo, 1);
}

/**
* transforms svg such that it is centered
*/
function center() {
    d3.select('svg')
        .call(zoom.translateTo, 0.5 * width, 0.5 * height);
}